from ..utils import rest
from starkcore.utils.resource import Resource


class PixDirector(Resource):
    """# PixDirector object
    Mandatory data that must be registered within the Central Bank for emergency contact purposes.
    When you initialize a PixDirector, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the list of created objects.
    ## Parameters (required):
    - name [string]: name of the PixDirector. ex: "Edward Stark"
    - tax_id [string]: tax ID (CPF) of the PixDirector. ex: "012.345.678-90"
    - phone [string]: phone of the PixDirector. ex: "+551198989898"
    - email [string]: email of the PixDirector. ex: "ned.stark@starkbank.com"
    - password [string]: password of the PixDirector. ex: "12345678"
    - team_email [string]: team email. ex: "pix.team@company.com"
    - team_phones [list of strings]: list of phones of the team. ex: ["+5511988889999", "+5511988889998"]
    ## Attributes (return-only):
    - status [string]: current PixDirector status. ex: "success"
    """

    def __init__(self, name, tax_id, phone, email, password, team_email, team_phones, id=None, status=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.tax_id = tax_id
        self.phone = phone
        self.email = email
        self.password = password
        self.team_email = team_email
        self.team_phones = team_phones
        self.status = status


_resource = {"class": PixDirector, "name": "PixDirector"}


def create(director, user=None):
    """# Create a PixDirector Object
    Send a PixDirector object for creation at the Stark Infra API
    ## Parameters (required):
    - director [list of PixDirector Object]: list of PixDirector objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixDirector object with updated attributes
    """
    return rest.post_single(resource=_resource, entity=director, user=user)



from starkcore.utils.resource import Resource


class CreditSigner(Resource):
    """# CreditSigner object
    CreditNote signer's information.
    ## Parameters (required):
    - name [string]: signer's name. ex: "Tony Stark"
    - contact [string]: signer's contact information. ex: "tony@starkindustries.com"
    - method [string]: delivery method for the contract. ex: "link"
    Attributes (return-only):
    - id [string]: unique id returned when the CreditSigner is created. ex: "5656565656565656"
    """

    def __init__(self, name, contact, method, id=None):
        Resource.__init__(self, id=id)
        self.name = name
        self.contact = contact
        self.method = method


_resource = {"class": CreditSigner, "name": "CreditSigner"}



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class PixInfraction(Resource):
    """# PixInfraction object
    PixInfractions are used to report transactions that are suspected of
    fraud, to request a refund or to reverse a refund.
    When you initialize a PixInfraction, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the created object.
    ## Parameters (required):
    - reference_id [string]: end_to_end_id or return_id of the transaction being reported. ex: "E20018183202201201450u34sDGd19lz"
    - type [string]: type of infraction report. Options: "reversal", "reversalChargeback"
    - method [string]:  method of Pix Infraction. Options: "scam", "unauthorized", "coercion", "invasion", "other", "unknown"
    ## Parameters (optional):
    - description [string, default None]: description for any details that can help with the infraction investigation.
    - tags [list of strings, default []]: list of strings for tagging. ex: ["travel", "food"]
    - fraud_type [string, default None]: type of Pix Fraud. Options: "identity", "mule", "scam", "unknown", "other"
    ## Attributes (return-only):
    - id [string]: unique id returned when the PixInfraction is created. ex: "5656565656565656"
    - fraud_id [string]: id of the Pix Fraud. ex: "5741774970552320"
    - bacen_id [string, default None]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - credited_bank_code [string]: bank_code of the credited Pix participant in the reported transaction. ex: "20018183"
    - debited_bank_code [string]: bank_code of the debited Pix participant in the reported transaction. ex: "20018183"
    - flow [string]: direction of the PixInfraction flow. Options: "out" if you created the PixInfraction, "in" if you received the PixInfraction.
    - analysis [string]: analysis that led to the result.
    - reported_by [string]: agent that reported the PixInfraction. Options: "debited", "credited"
    - result [string]: result after the analysis of the PixInfraction by the receiving party. Options: "agreed", "disagreed"
    - status [string]: current PixInfraction status. Options: "created", "failed", "delivered", "closed", "canceled"
    - created [datetime.datetime]: creation datetime for the PixInfraction. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the PixInfraction. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self,  reference_id, type, method, description=None, tags=None, fraud_type=None, id=None, fraud_id=None,
                 bacen_id=None, credited_bank_code=None, debited_bank_code=None, flow=None, analysis=None, reported_by=None,
                 result=None, status=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.reference_id = reference_id
        self.type = type
        self.method = method
        self.description = description
        self.tags = tags
        self.fraud_type = fraud_type
        self.fraud_id = fraud_id
        self.bacen_id = bacen_id
        self.credited_bank_code = credited_bank_code
        self.debited_bank_code = debited_bank_code
        self.flow = flow
        self.analysis = analysis
        self.reported_by = reported_by
        self.result = result
        self.status = status
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": PixInfraction, "name": "PixInfraction"}


def create(infractions, user=None):
    """# Create PixInfraction objects
    Create PixInfractions in the Stark Infra API
    ## Parameters (required):
    - infractions [list of PixInfractions]: list of PixInfraction objects to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixInfraction objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=infractions, user=user)


def get(id, user=None):
    """# Retrieve a PixInfraction object
    Retrieve the PixInfraction object linked to your Workspace in the Stark Infra API using its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixInfraction object that corresponds to the given id.
    """
    return rest.get_id(id=id, resource=_resource, user=user)


def query(limit=None, after=None, before=None, status=None, ids=None, bacen_id=None, type=None, flow=None, tags=None, user=None):
    """# Retrieve PixInfractions
    Receive a generator of PixInfraction objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. Options: ["created", "failed", "delivered", "closed", "canceled"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - bacen_id [string, default None]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - type [list of strings, default None]: filter for the type of retrieved PixInfractions. Options: "fraud", "reversal", "reversalChargeback"
    - flow [string, default None]: direction of the PixInfraction flow. Options: "out" if you created the PixInfraction, "in" if you received the PixInfraction.
    - tags [list of strings, default None]: list of strings for tagging. ex: ["travel", "food"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of PixInfraction objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        bacen_id=bacen_id,
        type=type,
        flow=flow,
        tags=tags,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, ids=None, bacen_id=None, type=None, flow=None, tags=None,
         user=None):
    """# Retrieve paged PixInfractions
    Receive a list of up to 100 PixInfraction objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call.
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. Options: ["created", "failed", "delivered", "closed", "canceled"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - bacen_id [string, default None]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - type [list of strings, default None]: filter for the type of retrieved PixInfractions. Options: "fraud", "reversal", "reversalChargeback"
    - flow [string, default None]: direction of the PixInfraction flow. Options: "out" if you created the PixInfraction, "in" if you received the PixInfraction.
    - tags [list of strings, default None]: list of strings for tagging. ex: ["travel", "food"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixInfraction objects with updated attributes and cursor to retrieve the next page of PixInfraction objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        bacen_id=bacen_id,
        type=type,
        flow=flow,
        tags=tags,
        user=user,
    )


def update(id, result, fraud_type=None, analysis=None, user=None):
    """# Update PixInfraction entity
    Update a PixInfraction by passing id.
    ## Parameters (required):
    - id [string]: PixInfraction id. ex: '5656565656565656'
    - result [string]: result after the analysis of the PixInfraction. Options: "agreed", "disagreed"
    ## Parameters (conditionally required):
    - fraud_type [string, default None]: type of Pix Fraud. Options: "identity", "mule", "scam", "unknown", "other"
    ## Parameters (optional):
    - analysis [string, default None]: analysis that led to the result.
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixInfraction with updated attributes
    """
    payload = {
        "result": result,
        "fraud_type": fraud_type,
        "analysis": analysis,
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def cancel(id, user=None):
    """# Cancel a PixInfraction entity
    Cancel a PixInfraction entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - canceled PixInfraction object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date, check_datetime
from ..utils import rest


class IssuingRestock(Resource):
    """# IssuingRestock object
    The IssuingRestock object displays the information of the restock orders created in your Workspace. 
    This resource place a restock order for a specific IssuingStock object.
    ## Parameters (required):
    - count [integer]: number of restocks to be restocked. ex: 100
    - stock_id [string]: IssuingStock unique id ex: "5136459887542272"
    ## Parameters (optional):
    - tags [list of strings, default None]: list of strings for tagging. ex: ["card", "corporate"]
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingRestock is created. ex: "5656565656565656"
    - status [string]: current IssuingRestock status. ex: "created", "processing", "confirmed"
    - updated [datetime.datetime]: latest update datetime for the IssuingRestock. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingRestock. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, count, stock_id, tags=None, id=None, status=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.count = count
        self.stock_id = stock_id
        self.tags = tags
        self.status = status
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": IssuingRestock, "name": "IssuingRestock"}


def create(restocks, user=None):
    """# Create IssuingRestocks
    Send a list of IssuingRestock objects for creation at the Stark Infra API
    ## Parameters (required):
    - restocks [list of IssuingRestock objects]: list of IssuingRestock objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingRestock objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=restocks, user=user)


def query(limit=None, after=None, before=None, status=None, stock_ids=None, ids=None, 
          tags=None, user=None):
    """# Retrieve IssuingRestocks
    Receive a generator of IssuingRestock objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "processing", "confirmed"]
    - stock_ids [list of string, default None]: list of stock_ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["card", "corporate"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of IssuingRestock objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        stock_ids=stock_ids,
        ids=ids,
        tags=tags,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, stock_ids=None, 
         ids=None, tags=None, user=None):
    """# Retrieve paged IssuingRestocks
    Receive a list of up to 100 IssuingRestock objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call.
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "processing", "confirmed"]
    - stock_ids [list of string, default None]: list of stock_ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["card", "corporate"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingRestock objects with updated attributes
    - cursor to retrieve the next page of IssuingRestock objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        stock_ids=stock_ids,
        ids=ids,
        tags=tags,
        user=user,
    )


def get(id, user=None):
    """# Retrieve a specific IssuingRestock
    Receive a single IssuingRestock object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingRestock object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)



from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ...utils import rest
from ..__issuingpurchase import _resource as _issuing_purchase_resource
from starkcore.error import Error


class Log(Resource):
    """# issuingpurchase.Log object
    Every time an IssuingPurchase entity is updated, a corresponding issuingpurchase.Log
    is generated for the entity. This log is never generated by the
    user, but it can be retrieved to check additional information
    on the IssuingPurchase.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - purchase [IssuingPurchase]: IssuingPurchase entity to which the log refers to.
    - issuing_transaction_id [string]: transaction ID related to the IssuingCard.
    - errors [list of StarkCore.Error]: list of errors linked to this IssuingPurchase event.
    - type [string]: type of the IssuingPurchase event which triggered the log creation. ex: "approved", "canceled", "confirmed", "denied", "reversed", "voided"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, purchase, issuing_transaction_id, errors, type, created):
        Resource.__init__(self, id=id)

        self.purchase = from_api_json(_issuing_purchase_resource, purchase)
        self.issuing_transaction_id = issuing_transaction_id
        self.errors = _parse_errors(errors)
        self.type = type
        self.created = check_datetime(created)


_resource = {"class": Log, "name": "IssuingPurchaseLog"}


def get(id, user=None):
    """# Retrieve a specific issuingpurchase.Log
    Receive a single issuingpurchase.Log object previously created by the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - issuingpurchase.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(ids=None, limit=None, after=None, before=None, types=None, purchase_ids=None, user=None):
    """# Retrieve issuingpurchase.Log
    Receive a generator of issuingpurchase.Log objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["approved", "canceled", "confirmed", "denied", "reversed", "voided"]
    - purchase_ids [list of strings, default None]: list of Purchase ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of IssuingPurchase ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of issuingpurchase.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        purchase_ids=purchase_ids,
        user=user,
    )


def page(cursor=None, ids=None, limit=None, after=None, before=None, types=None, purchase_ids=None, user=None):
    """# Retrieve paged issuingpurchase.Log
    Receive a list of up to 100 issuingpurchase.Log objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["approved", "canceled", "confirmed", "denied", "reversed", "voided"]
    - purchase_ids [list of strings, default None]: list of Purchase ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of IssuingPurchase ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of issuingpurchase.Log objects with updated attributes
    - cursor to retrieve the next page of issuingpurchase.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        purchase_ids=purchase_ids,
        user=user,
    )

def _parse_errors(errors):
    parsed_errors = []
    for error in errors:
        if isinstance(error, Error):
            parsed_errors.append(error)
            continue
        parsed_errors.append(Error(code=error['code'], message=error['message']))
    return parsed_errors



from starkcore.utils.subresource import SubResource


class Description(SubResource):
    """# creditnote.invoice.Description object
    Invoice description information.
    ## Parameters (required):
    - key [string]: Description for the value. ex: "Taxes"
    ## Parameters (optional):
    - value [string, default ""]: amount related to the described key. ex: "R$100,00"
    """

    def __init__(self, key, value=None):
        self.key = key
        self.value = value


resource = {"class": Description, "name": "Description"}



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class IndividualIdentity(Resource):
    """# IndividualIdentity object
    An IndividualDocument represents an individual to be validated. It can have several individual documents attached
    to it, which are used to validate the identity of the individual. Once an individual identity is created, individual
    documents must be attached to it using the created method of the individual document resource. When all the required
    individual documents are attached to an individual identity it can be sent to validation by patching its status to 
    processing.
    When you initialize a IndividualIdentity, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the list of created objects.
    ## Parameters (required):
    - name [string]: individual's full name. ex: "Edward Stark".
    - tax_id [string]: individual's tax ID (CPF). ex: "594.739.480-42"
    ## Parameters (optional):
    - tags [list of strings, default []]: list of strings for reference when searching for IndividualIdentities. ex: ["employees", "monthly"]
    ## Attributes (return-only):
    - id [string]: unique id returned when the IndividualIdentity is created. ex: "5656565656565656"
    - status [string]: current status of the IndividualIdentity. ex: "created", "canceled", "processing", "failed", "success"
    - created [datetime.datetime]: creation datetime for the IndividualIdentity. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, name, tax_id, tags=None, id=None, status=None, created=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.tax_id = tax_id
        self.tags = tags
        self.status = status
        self.created = check_datetime(created)


_resource = {"class": IndividualIdentity, "name": "IndividualIdentity"}


def create(identities, user=None):
    """# Create IndividualIdentities
    Send a list of IndividualIdentity objects for creation at the Stark Infra API
    ## Parameters (required):
    - identities [list of IndividualIdentity objects]: list of IndividualIdentity objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IndividualIdentity objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=identities, user=user)


def get(id, user=None):
    """# Retrieve a specific IndividualIdentity
    Receive a single IndividualIdentity object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IndividualIdentity object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve IndividualIdentities
    Receive a generator of IndividualIdentity objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "canceled", "processing", "failed", "success"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of IndividualIdentity objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve paged IndividualIdentities
    Receive a list of up to 100 IndividualIdentity objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "canceled", "processing", "failed", "success"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IndividualIdentity objects with updated attributes
    - cursor to retrieve the next page of IndividualIdentity objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def update(id, status, user=None):
    """# Update IndividualIdentity entity
    Update an IndividualIdentity by passing id.
    ## Parameters (required):
    - id [string]: IndividualIdentity id. ex: '5656565656565656'
    - status [string]: You may send IndividualDocuments to validation by passing 'processing' in the status
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - target IndividualIdentity with updated attributes
    """
    payload = {
        "status": status
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def cancel(id, user=None):
    """# Cancel a IndividualIdentity entity
    Cancel a IndividualIdentity entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: IndividualIdentity unique id. ex: "6306109539221504"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - canceled IndividualIdentity object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..utils import rest


class StaticBrcode(Resource):
    """# StaticBrcode object
    A StaticBrcode stores account information in the form of a PixKey and can be used to create 
    Pix transactions easily.
    When you initialize a StaticBrcode, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    ## Parameters (required):
    - name [string]: receiver's name. ex: "Tony Stark"
    - key_id [string]: receiver's Pixkey id. ex: "+5541999999999"
    - city [string]: receiver's city name. ex: "Rio de Janeiro"
    ## Parameters (optional):
    - amount [integer, default 0]: positive integer that represents the amount in cents of the resulting Pix transaction. ex: 1234 (= R$ 12.34)
    - cashier_bank_code [string, default None]: Cashier's bank code. ex: "20018183".
    - reconciliation_id [string, default None]: id to be used for conciliation of the resulting Pix transaction. This id must have up to 25 alphanumeric characters ex: "ah27s53agj6493hjds6836v49"
    - description [string, default None]: optional description to override default description to be shown in the bank statement. ex: "Payment for service #1234"
    - tags [list of strings, default []]: list of strings for tagging. ex: ["travel", "food"]
    ## Attributes (return-only):
    - id [string]: id returned on creation, this is the BR code. ex: "00020126360014br.gov.bcb.pix0114+552840092118152040000530398654040.095802BR5915Jamie Lannister6009Sao Paulo620705038566304FC6C"
    - uuid [string]: unique uuid returned when a StaticBrcode is created. ex: "97756273400d42ce9086404fe10ea0d6"
    - url [string]: url link to the BR code image. ex: "https://brcode-h.development.starkinfra.com/static-qrcode/97756273400d42ce9086404fe10ea0d6.png"
    - updated [datetime.datetime]: latest update datetime for the StaticBrcode. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the StaticBrcode. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, name, key_id, city, amount=None, cashier_bank_code=None, reconciliation_id=None, tags=None,
                 id=None, description=None, uuid=None, url=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.key_id = key_id
        self.city = city
        self.amount = amount
        self.cashier_bank_code = cashier_bank_code
        self.reconciliation_id = reconciliation_id
        self.description = description
        self.tags = tags
        self.uuid = uuid
        self.url = url
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": StaticBrcode, "name": "StaticBrcode"}


def create(brcodes, user=None):
    """# Create StaticBrcodes
    Send a list of StaticBrcode objects for creation at the Stark Infra API
    ## Parameters (required):
    - brcodes [list of StaticBrcode objects]: list of StaticBrcode objects to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of StaticBrcode objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=brcodes, user=user)


def get(uuid, user=None):
    """# Retrieve a specific StaticBrcode
    Receive a single StaticBrcode object previously created in the Stark Infra API by its uuid
    ## Parameters (required):
    - uuid [string]: object's unique uuid. ex: "97756273400d42ce9086404fe10ea0d6"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - StaticBrcode object with updated attributes
    """
    return rest.get_id(resource=_resource, id=uuid, user=user)


def query(limit=None, after=None, before=None, uuids=None, tags=None, user=None):
    """# Retrieve StaticBrcodes
    Receive a generator of StaticBrcode objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - uuids [list of strings, default None]: list of uuids to filter retrieved objects. ex: ["97756273400d42ce9086404fe10ea0d6", "e3da0b6d56fa4045b9b295b2be82436e"]
    - tags [list of strings, default None]: list of tags to filter retrieved objects. ex: ["travel", "food"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of StaticBrcode objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        uuids=uuids,
        tags=tags,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, uuids=None, tags=None, user=None):
    """# Retrieve paged StaticBrcodes
    Receive a list of up to 100 StaticBrcode objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - uuids [list of strings, default None]: list of uuids to filter retrieved objects. ex: ["97756273400d42ce9086404fe10ea0d6", "e3da0b6d56fa4045b9b295b2be82436e"]
    - tags [list of strings, default None]: list of tags to filter retrieved objects. ex: ["travel", "food"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of StaticBrcode objects with updated attributes
    - cursor to retrieve the next page of StaticBrcode objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        uuids=uuids,
        tags=tags,
        user=user,
    )



from ..utils import rest
from ..utils.parse import parse_and_verify
from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..creditnote.log.__log import _resource as _creditnote_log_resource
from ..pixkey.log.__log import _resource as _pixkey_log_resource
from ..pixclaim.log.__log import _resource as _pixclaim_log_resource
from ..pixrequest.log.__log import _resource as _pixrequest_log_resource
from ..pixreversal.log.__log import _resource as _pixreversal_log_resource
from ..pixchargeback.log.__log import _resource as _pixchargeback_log_resource
from ..pixinfraction.log.__log import _resource as _pixinfraction_log_resource
from ..issuingcard.log.__log import _resource as _issuingcard_log_resource
from ..issuinginvoice.log.__log import _resource as _issuinginvoice_log_resource
from ..issuingpurchase.log.__log import _resource as _issuingpurchase_log_resource


_resource_by_subscription = {
    "pix-key": _pixkey_log_resource,
    "pix-claim": _pixclaim_log_resource,
    "pix-chargeback": _pixchargeback_log_resource,
    "pix-infraction": _pixinfraction_log_resource,
    "pix-request.in": _pixrequest_log_resource,
    "pix-request.out": _pixrequest_log_resource,
    "pix-reversal.in": _pixreversal_log_resource,
    "pix-reversal.out": _pixreversal_log_resource,
    "issuing-card": _issuingcard_log_resource,
    "issuing-invoice": _issuinginvoice_log_resource,
    "issuing-purchase": _issuingpurchase_log_resource,
    "credit-note": _creditnote_log_resource,
}


class Event(Resource):
    """# Webhook Event object
    An Event is the notification received from the subscription to the Webhook.
    Events cannot be created, but may be retrieved from the Stark Infra API to
    list all generated updates on entities.
    ## Attributes (return-only):
    - id [string]: unique id returned when the Event is created. ex: "5656565656565656"
    - log [Log]: a Log object from one of the subscribed services (PixRequestLog, PixReversalLog)
    - created [datetime.datetime]: creation datetime for the notification Event. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - is_delivered [bool]: true if the Event has been successfully delivered to the user url. ex: False
    - subscription [string]: service that triggered this Event. Options: "pix-request.in", "pix-request.out", "pix-reversal.in", "pix-reversal.out", "pix-key", "pix-claim", "pix-infraction", "pix-chargeback", "issuing-card", "issuing-invoice", "issuing-purchase", "credit-note"
    - workspace_id [string]: ID of the Workspace that generated this Event. Mostly used when multiple Workspaces have Webhooks registered to the same endpoint. ex: "4545454545454545"
    """

    def __init__(self, log, created, is_delivered, subscription, workspace_id, id):
        Resource.__init__(self, id=id)

        self.log = log
        self.created = check_datetime(created)
        self.is_delivered = is_delivered
        self.subscription = subscription
        self.workspace_id = workspace_id
        if subscription in _resource_by_subscription:
            self.log = from_api_json(resource=_resource_by_subscription[subscription], json=log)


_resource = {"class": Event, "name": "Event"}


def get(id, user=None):
    """# Retrieve a specific notification Event
    Receive a single notification Event object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - Event object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, is_delivered=None, user=None):
    """# Retrieve notification Events
    Receive a generator of notification Event objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - is_delivered [bool, default None]: bool to filter successfully delivered events. ex: True or False
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of Event objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        is_delivered=is_delivered,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, is_delivered=None, user=None):
    """# Retrieve paged Events
    Receive a list of up to 100 Event objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None]: date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - is_delivered [bool, default None]: bool to filter successfully delivered events. ex: True or False
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of Event objects with updated attributes
    - cursor to retrieve the next page of Event objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        is_delivered=is_delivered,
        user=user,
    )


def delete(id, user=None):
    """# Delete a Webhook Event entity
    Delete a notification Event entity previously created in the Stark Infra API by its ID
    ## Parameters (required):
    - id [string]: Event unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - deleted Event object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)


def update(id, is_delivered, user=None):
    """# Update notification Event entity
    Update notification Event by passing id.
    If is_delivered is True, the event will no longer be returned on queries with is_delivered=False.
    ## Parameters (required):
    - id [list of strings]: Event unique ids. ex: "5656565656565656"
    - is_delivered [bool]: If True and event hasn't been delivered already, event will be set as delivered. ex: True
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - target Event with updated attributes
    """
    payload = {
        "is_delivered": is_delivered
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def parse(content, signature, user=None):
    """# Create a single notification Event from a content string
    Create a single Event object received from Event listening at subscribed user endpoint.
    If the provided digital signature does not check out with the StarkInfra public key, a
    starkinfra.error.InvalidSignatureError will be raised.
    ## Parameters (required):
    - content [string]: response content from request received at user endpoint (not parsed)
    - signature [string]: base-64 digital signature received at response header "Digital-Signature"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - Parsed Event object
    """

    return parse_and_verify(
        content=content,
        signature=signature,
        user=user,
        resource=_resource,
        key="event",
    )



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class PixFraud(Resource):
    """# PixFraud object
    PixFrauds are used to report a PixKey or taxId when a fraud
    has been confirmed.
    When you initialize a PixFraud, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the created object.
    ## Parameters (required):
    - external_id [string]: end_to_end_id or return_id of the transaction being reported. ex: "my_external_id"
    - type [string]: type of PixFraud. Options: "identity", "mule", "scam", "other"
    - tax_id [string]: user tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    ## Parameters (optional):
    - key_id [string]: marked PixKey id. ex: "+5511989898989"
    - tags [list of strings, default []]: list of strings for tagging. ex: ["fraudulent"]
    ## Attributes (return-only):
    - id [string]: unique id returned when the PixFraud is created. ex: "5656565656565656"
    - bacen_id [string]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - status [string]: current PixFraud status. Options: "created", "failed", "registered", "canceled".
    - created [string]: creation datetime for the PixFraud. ex: "2020-03-10 10:30:00.000000+00:00"
    - updated [string]: latest update datetime for the PixFraud. ex: "2020-03-10 10:30:00.000000+00:00"
    """

    def __init__(self,  external_id, type, tax_id, key_id=None, tags=None, id=None,
                 bacen_id=None, status=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.external_id = external_id
        self.type = type
        self.tax_id = tax_id
        self.key_id = key_id
        self.tags = tags
        self.bacen_id = bacen_id
        self.status = status
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": PixFraud, "name": "PixFraud"}


def create(frauds, user=None):
    """# Create PixFrauds objects
    Create PixFrauds in the Stark Infra API
    ## Parameters (required):
    - frauds [list of PixFrauds]: list of PixFraud objects to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixFraud objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=frauds, user=user)


def get(id, user=None):
    """# Retrieve a PixFraud object
    Retrieve the PixFraud object linked to your Workspace in the Stark Infra API using its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixFraud object that corresponds to the given id.
    """
    return rest.get_id(id=id, resource=_resource, user=user)


def query(limit=None, after=None, before=None, status=None, ids=None, bacen_id=None, type=None, flow=None, tags=None, user=None):
    """# Retrieve PixFrauds
    Receive a generator of PixFraud objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. Options: ["created", "failed", "registered", "canceled"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - bacen_id [string, default None]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - type [list of strings, default None]: filter for the type of retrieved PixFrauds. Options: "reversal", "reversalChargeback"
    - tags [list of strings, default None]: list of strings for tagging. ex: ["fraudulent"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of PixFraud objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        bacen_id=bacen_id,
        type=type,
        flow=flow,
        tags=tags,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, ids=None, bacen_id=None, type=None, flow=None, tags=None,
         user=None):
    """# Retrieve paged PixFraud
    Receive a list of up to 100 PixFraud objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call.
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. Options: ["created", "failed", "registered", "canceled"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - bacen_id [string, default None]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - type [list of strings, default None]: filter for the type of retrieved PixFrauds. Options: "reversal", "reversalChargeback"
    - tags [list of strings, default None]: list of strings for tagging. ex: ["fraudulent"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixFraud objects with updated attributes and cursor to retrieve the next page of PixFraud objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        bacen_id=bacen_id,
        type=type,
        flow=flow,
        tags=tags,
        user=user,
    )


def cancel(id, user=None):
    """# Cancel a PixFraud entity
    Cancel a PixFraud entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: PixFraud unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - canceled PixFraud object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)




from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime


class PixBalance(Resource):
    """# PixBalance object
    The PixBalance object displays the current balance of the Workspace, which
    is the result of the sum of all transactions within this Workspace.
    The balance is never generated by the user, but it can be retrieved to
    see the available information.
    ## Attributes (return-only):
    - id [string]: unique id returned when Balance is created. ex: "5656565656565656"
    - amount [integer]: current balance amount of the Workspace in cents. ex: 200 (= R$ 2.00)
    - currency [string]: currency of the current Workspace. Expect others to be added eventually. ex: "BRL"
    - updated [datetime.datetime]: latest update datetime for the balance. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id=None, amount=None, currency=None, updated=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.currency = currency
        self.updated = check_datetime(updated)


_resource = {"class": PixBalance, "name": "PixBalance"}


def get(user=None):
    """# Retrieve the PixBalance object
    Receive the PixBalance object linked to your Workspace in the Stark Infra API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixBalance object with updated attributes
    """
    return next(rest.get_stream(resource=_resource, user=user))



from ..utils import rest
from starkcore.utils.resource import Resource


class PixUser(Resource):
    """# PixUser object
    Pix Users are used to get fraud statistics of a user.
    ## Parameters (required):
    - id [string]: user tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    ## Attributes (return-only):
    - statistics [list of PixUser.Statistics, default []]: list of PixUser.Statistics objects. ex: [PixUser.Statistics(after="2023-11-06T18:57:08.325090+00:00", source="pix-key")]
    """

    def __init__(self, id, statistics=None):
        Resource.__init__(self, id=id)

        self.statistics = statistics


_resource = {"class": PixUser, "name": "PixUser"}


def get(id, key_id=None, user=None):
    """# Retrieve a PixUser object
    Receive a single PixUser object information by passing its taxId
    ## Parameters (required):
    - id [string]: user tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    ## Parameters (optional):
    - key_id [string]: marked PixKey id. ex: "+5511989898989"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixUser object that corresponds to the given id.
    """
    return rest.get_id(id=id, resource=_resource, key_id=key_id, user=user)



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..utils import rest
from ..issuingrule import parse_rules


class IssuingHolder(Resource):
    """# IssuingHolder object
    The IssuingHolder describes a card holder that may group several cards.
    ## Parameters (required):
    - name [string]: card holder name. ex: "Tony Stark"
    - tax_id [string]: card holder tax ID ex: "012.345.678-90"
    - external_id [string] card holder unique id, generated by the user to avoid duplicated holders. ex: "my-entity/123"
    ## Parameters (optional):
    - rules [list of IssuingRule, default []]: [EXPANDABLE] list of holder spending rules
    - tags [list of strings, default []]: list of strings for tagging. ex: ["travel", "food"]
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingHolder is created. ex: "5656565656565656"
    - status [string]: current IssuingHolder status. ex: "active", "blocked", "canceled"
    - created [datetime.datetime]: creation datetime for the IssuingHolder. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the IssuingHolder. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, name, tax_id, external_id, rules=None, tags=None, id=None, status=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.tax_id = tax_id
        self.external_id = external_id
        self.rules = parse_rules(rules)
        self.tags = tags
        self.status = status
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": IssuingHolder, "name": "IssuingHolder"}


def create(holders, expand=None, user=None):
    """# Create IssuingHolder
    Send a list of IssuingHolder objects for creation at the Stark Infra API
    ## Parameters (required):
    - holders [list of IssuingHolder objects]: list of IssuingHolder objects to be created in the API
    ## Parameters (optional):
    - expand [list of strings, default None]: fields to expand information. Options: ["rules"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of IssuingHolder objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=holders, expand=expand, user=user)


def get(id, expand=None, user=None):
    """# Retrieve a specific IssuingHolder
    Receive a single IssuingHolder object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - expand [list of strings, default None]: fields to expand information. Options: ["rules"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - IssuingHolder object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, expand=expand, user=user)


def query(limit=None, ids=None, after=None, before=None, status=None, tags=None, expand=None, user=None):
    """# Retrieve IssuingHolders
    Receive a generator of IssuingHolder objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["active", "blocked", "canceled"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - expand [string, default None]: fields to expand information. Options: ["rules"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of IssuingHolder objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        ids=ids,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        expand=expand,
        user=user,
    )


def page(cursor=None, limit=None, ids=None, after=None, before=None, status=None, tags=None, expand=None, user=None):
    """# Retrieve IssuingHolders
    Receive a list of IssuingHolder objects previously created in the Stark Infra API and the cursor to the next page.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["active", "blocked", "canceled"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - expand [string, default None]: fields to expand information. Options: ["rules"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of IssuingHolder objects with updated attributes
    - cursor to retrieve the next page of IssuingHolder objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        ids=ids,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        expand=expand,
        user=user,
    )


def update(id, status=None, name=None, rules=None, tags=None, user=None):
    """# Update IssuingHolder entity
    Update an IssuingHolder by passing id, if it hasn't been paid yet.
    ## Parameters (required):
    - id [string]: IssuingHolder id. ex: '5656565656565656'
    ## Parameters (optional):
    - status [string]: You may block the IssuingHolder by passing 'blocked' in the status
    - name [string]: card holder name. ex: "Tony Stark"
    - tags [list of strings]: list of strings for tagging. ex: ["tony", "stark"]
    - rules [list of dictionaries, default None]: list of dictionaries with "amount": int, "currencyCode": string, "id": string, "interval": string, "name": string pairs
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - target IssuingHolder with updated attributes
    """
    payload = {
        "status": status,
        "name": name,
        "rules": rules,
        "tags": tags,
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def cancel(id, user=None):
    """# Cancel an IssuingHolder entity
    Cancel an IssuingHolder entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: IssuingHolder unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - canceled IssuingHolder object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime
from ..utils import rest


class IssuingDesign(Resource):
    """# IssuingDesign object
    The IssuingDesign object displays information on the card and card package designs available to your Workspace.
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingDesign is created. ex: "5656565656565656"
    - name [string]: card or package design name. ex: "stark-plastic-dark-001"
    - embosser_ids [list of strings]: list of embosser unique ids. ex: ["5136459887542272", "5136459887542273"]
    - type [string]: card or package design type. Options: "card", "envelope"
    - updated [datetime.datetime]: latest update datetime for the IssuingDesign. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingDesign. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id=None, name=None, embosser_ids=None, type=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.embosser_ids = embosser_ids
        self.type = type
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": IssuingDesign, "name": "IssuingDesign"}


def parse_designs(designs):
    parsed_designs = []
    if designs is None:
        return designs
    for design in designs:
        if isinstance(design, IssuingDesign):
            parsed_designs.append(design)
            continue
        parsed_designs.append(from_api_json(_resource, design))
    return parsed_designs


def query(limit=None, ids=None, user=None):
    """# Retrieve IssuingDesigns
    Receive a generator of IssuingDesign objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of IssuingDesign objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, ids=None, user=None):
    """# Retrieve paged IssuingDesigns
    Receive a list of up to 100 IssuingDesign objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingDesign objects with updated attributes
    - cursor to retrieve the next page of IssuingDesign objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        ids=ids,
        user=user,
    )


def get(id, user=None):
    """# Retrieve a specific IssuingDesign
    Receive a single IssuingDesign object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingDesign object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def pdf(id, user=None):
    """# Retrieve a specific IssuingDesign pdf file
    Receive a single IssuingDesign pdf file generated in the Stark Infra API by its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - IssuingDesign pdf file
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="pdf")



from ..utils.parse import parse_and_verify
from starkcore.utils.subresource import SubResource


class IssuingTokenActivation(SubResource):
    """# IssuingTokenActivation object
    The IssuingTokenActivation object displays the necessary information to proceed with the card tokenization.
    You will receive this object at your registered URL to notify you which method your user want to receive the activation code.
    The POST request must be answered with no content, within 2 seconds, and with an HTTP status code 200.
    After that, you may generate the activation code and send it to the cardholder.
    ## Attributes (return-only):
    - card_id [string]: card ID which the token is bounded to. ex: "5656565656565656"
    - token_id [string]: token unique id. ex: "5656565656565656" 
    - tags [list of strings]: tags to filter retrieved object. ex: ["tony", "stark"]
    - activation_method [dictionary]: dictionary object with "type":string and "value":string pairs
    """

    def __init__(self, card_id=None, token_id=None, tags=None, activation_method=None):
        self.card_id = card_id
        self.token_id = token_id
        self.tags = tags
        self.activation_method = activation_method


_resource = {"class": IssuingTokenActivation, "name": "IssuingTokenActivation"}


def parse(content, signature, user=None):
    """# Create a single verified IssuingTokenActivation request from a content string
    Use this method to parse and verify the authenticity of the request received at the informed endpoint.
    Activation requests are posted to your registered endpoint whenever IssuingTokenActivations are received.
    If the provided digital signature does not check out with the StarkInfra public key, a stark.exception.InvalidSignatureException will be raised.
    ## Parameters (required):
    - content [string]: response content from request received at user endpoint (not parsed)
    - signature [string]: base-64 digital signature received at response header "Digital-Signature"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - Parsed IssuingTokenActivation object
    """
    return parse_and_verify(
        content=content,
        signature=signature,
        user=user,
        resource=_resource,
        key="",
    )



from json import dumps
from ..utils import rest
from ..utils.parse import parse_and_verify
from starkcore.utils.api import api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class PixRequest(Resource):
    """# PixRequest object
    PixRequests are used to receive or send instant payments to accounts
    hosted in any Pix participant.
    When you initialize a PixRequest, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the list of created objects.
    ## Parameters (required):
    - amount [integer]: amount in cents to be transferred. ex: 11234 (= R$ 112.34)
    - external_id [string]: string that must be unique among all your PixRequests. Duplicated external IDs will cause failures. By default, this parameter will block any PixRequests that repeats amount and receiver information on the same date. ex: "my-internal-id-123456"
    - sender_name [string]: sender's full name. ex: "Edward Stark"
    - sender_tax_id [string]: sender's tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - sender_branch_code [string]: sender's bank account branch code. Use '-' in case there is a verifier digit. ex: "1357-9"
    - sender_account_number [string]: sender's bank account number. Use '-' before the verifier digit. ex: "876543-2"
    - sender_account_type [string]: sender's bank account type. ex: "checking", "savings", "salary" or "payment"
    - receiver_name [string]: receiver's full name. ex: "Edward Stark"
    - receiver_tax_id [string]: receiver's tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - receiver_bank_code [string]: receiver's bank institution code in Brazil. ex: "20018183"
    - receiver_account_number [string]: receiver's bank account number. Use '-' before the verifier digit. ex: "876543-2"
    - receiver_branch_code [string]: receiver's bank account branch code. Use '-' in case there is a verifier digit. ex: "1357-9"
    - receiver_account_type [string]: receiver's bank account type. ex: "checking", "savings", "salary" or "payment"
    - end_to_end_id [string]: central bank's unique transaction ID. ex: "E79457883202101262140HHX553UPqeq"
    ## Parameters (conditionally-required):
    - cashier_type [string]: Cashier's type. Required if the cash_amount is different from 0. Options: "merchant", "participant" and "other"
    - cashier_bank_code [string]: Cashier's bank code. Required if the cash_amount is different from 0. ex: "20018183"
    ## Parameters (optional):
    - cash_amount [integer]: Amount to be withdrawn from the cashier in cents. ex: 1000 (= R$ 10.00)
    - receiver_key_id [string, default None]: receiver's dict key. ex: "20.018.183/0001-80"
    - description [string, default None]: optional description to override default description to be shown in the bank statement. ex: "Payment for service #1234"
    - reconciliation_id [string, default None]: Reconciliation ID linked to this payment. ex: "b77f5236-7ab9-4487-9f95-66ee6eaf1781"
    - initiator_tax_id [string, default None]: Payment initiator's tax id (CPF/CNPJ). ex: "01234567890" or "20.018.183/0001-80"
    - tags [list of strings, default []]: list of strings for reference when searching for PixRequests. ex: ["employees", "monthly"]
    - method [string, default None]: execution  method for thr creation of the Pix. ex: "manual", "payerQrcode", "dynamicQrcode"
    ## Attributes (return-only):
    - id [string]: unique id returned when the PixRequest is created. ex: "5656565656565656"
    - fee [integer]: fee charged when PixRequest is paid. ex: 200 (= R$ 2.00)
    - status [string]: current PixRequest status. ex: "created", "processing", "success", "failed"
    - flow [string]: direction of money flow. ex: "in" or "out"
    - sender_bank_code [string]: sender's bank institution code in Brazil. ex: "20018183"
    - created [datetime.datetime]: creation datetime for the PixRequest. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the PixRequest. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, external_id, sender_name, sender_tax_id, sender_branch_code,
                 sender_account_number, sender_account_type, receiver_name, receiver_tax_id, receiver_bank_code,
                 receiver_account_number, receiver_branch_code, receiver_account_type, end_to_end_id,
                 cashier_type=None, cashier_bank_code=None, cash_amount=None, receiver_key_id=None, description=None, 
                 reconciliation_id=None, initiator_tax_id=None, tags=None, method=None, id=None, fee=None,
                 status=None, flow=None, sender_bank_code=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.external_id = external_id
        self.sender_name = sender_name
        self.sender_tax_id = sender_tax_id
        self.sender_branch_code = sender_branch_code
        self.sender_account_number = sender_account_number
        self.sender_account_type = sender_account_type
        self.receiver_name = receiver_name
        self.receiver_tax_id = receiver_tax_id
        self.receiver_bank_code = receiver_bank_code
        self.receiver_account_number = receiver_account_number
        self.receiver_branch_code = receiver_branch_code
        self.receiver_account_type = receiver_account_type
        self.end_to_end_id = end_to_end_id
        self.cashier_type = cashier_type
        self.cashier_bank_code = cashier_bank_code
        self.cash_amount = cash_amount
        self.receiver_key_id = receiver_key_id
        self.description = description
        self.reconciliation_id = reconciliation_id
        self.initiator_tax_id = initiator_tax_id
        self.tags = tags
        self.method = method
        self.fee = fee
        self.status = status
        self.flow = flow
        self.sender_bank_code = sender_bank_code
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": PixRequest, "name": "PixRequest"}


def create(requests, user=None):
    """# Create PixRequests
    Send a list of PixRequest objects for creation at the Stark Infra API
    ## Parameters (required):
    - requests [list of PixRequest objects]: list of PixRequest objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixRequest objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=requests, user=user)


def get(id, user=None):
    """# Retrieve a specific PixRequest
    Receive a single PixRequest object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixRequest object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, status=None, ids=None, end_to_end_ids=None,
          external_ids=None, tags=None, user=None):
    """# Retrieve PixRequests
    Receive a generator of PixRequest objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "processing", "success", "failed"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - end_to_end_ids [list of strings, default None]: central bank's unique transaction IDs. ex: ["E79457883202101262140HHX553UPqeq", "E79457883202101262140HHX553UPxzx"]
    - external_ids [list of strings, default None]: url safe strings that must be unique among all your PixRequests. Duplicated external IDs will cause failures. By default, this parameter will block any PixRequests that repeats amount and receiver information on the same date. ex: ["my-internal-id-123456", "my-internal-id-654321"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of PixRequest objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        end_to_end_ids=end_to_end_ids,
        external_ids=external_ids,
        tags=tags,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, ids=None,
         end_to_end_ids=None, external_ids=None, tags=None, user=None):
    """# Retrieve paged PixRequests
    Receive a list of up to 100 PixRequest objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "processing", "success", "failed"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - end_to_end_ids [list of strings, default None]: central bank's unique transaction IDs. ex: ["E79457883202101262140HHX553UPqeq", "E79457883202101262140HHX553UPxzx"]
    - external_ids [list of strings, default None]: url safe strings that must be unique among all your PixRequests. Duplicated external IDs will cause failures. By default, this parameter will block any PixRequests that repeats amount and receiver information on the same date. ex: ["my-internal-id-123456", "my-internal-id-654321"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixRequest objects with updated attributes
    - cursor to retrieve the next page of PixRequest objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        end_to_end_ids=end_to_end_ids,
        external_ids=external_ids,
        tags=tags,
        user=user,
    )


def parse(content, signature, user=None):
    """# Create a single verified PixRequest object from a content string
    Create a single PixRequest object from a content string received from a handler listening at the request url.
    If the provided digital signature does not check out with the StarkInfra public key, a
    starkinfra.error.InvalidSignatureError will be raised.
    ## Parameters (required):
    - content [string]: response content from request received at user endpoint (not parsed)
    - signature [string]: base-64 digital signature received at response header "Digital-Signature"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - Parsed PixRequest object
    """
    request = parse_and_verify(
        content=content,
        signature=signature,
        user=user,
        resource=_resource
    )

    request.fee = request.fee or 0
    request.tags = request.tags or []
    request.external_id = request.external_id or ""
    request.description = request.description or ""
    
    return request


def response(status, reason=None):
    """# Helps you respond to a PixRequest authorization.
    Authorization requests will be posted at your registered
    endpoint whenever inbound PixRequests are received.
    ## Parameters (required):
    - status [string]: response to the authorization. ex: "approved" or "denied"
    ## Parameters (conditionally required):
    - reason [string, default None]: denial reason. Required if the status is "denied". Options: "invalidAccountNumber", "blockedAccount", "accountClosed", "invalidAccountType", "invalidTransactionType", "taxIdMismatch", "invalidTaxId", "orderRejected", "reversalTimeExpired", "settlementFailed"
    ## Return:
    - Dumped JSON string that must be returned to us
    """
    params = {
        "authorization": {
            "status": status,
            "reason": reason,
        }
    }
    return dumps(api_json(params))



from json import dumps
from starkinfra.utils import rest
from starkcore.utils.api import api_json
from ..utils.parse import parse_and_verify
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class IssuingToken(Resource):
    """# IssuingToken object
    The IssuingToken object displays the information of the tokens created in your Workspace.
    ## Attributes (return-only):
    - card_id [string]: card ID which the token is bounded to. ex: "5656565656565656"
    - wallet_id [string]: wallet provider which the token is bounded to. ex: "google"
    - wallet_name [string]: wallet name. ex: "GOOGLE"
    - merchant_id [string]: merchant unique id. ex: "5656565656565656"
    ## Attributes (IssuingToken only):
    - id [string]: unique id returned when IssuingToken is created. ex: "5656565656565656"
    - external_id [string]: a unique string among all your IssuingTokens, used to avoid resource duplication. ex: "DSHRMC00002626944b0e3b539d4d459281bdba90c2588791"
    - tags [list of strings]: list of strings for reference when searching for IssuingToken. ex: ["employees", "monthly"]
    - status [string]: current IssuingToken status. ex: "active", "blocked", "canceled", "frozen" or "pending"
    - updated [datetime.datetime]: latest update datetime for the IssuingToken. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingToken. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    ## Attributes (authorization request only):
    - activation_code [string]: activation code recived through the bank app or sms. ex: "481632" 
    - method_code [string]: provisioning method. Options: "app", "token", "manual", "server" or "browser"
    - device_type [string]: device type used for tokenization. ex: "Phone"
    - device_name [string]: device name used for tokenization. ex: "My phone" 
    - device_serial_number [string]: device serial number used for tokenization. ex: "2F6D63"
    - device_os_name [string]: device operational system name used for tokenization. ex: "Android"
    - device_os_version [string]: device operational system version used for tokenization. ex: "4.4.4"
    - device_imei [string]: device imei used for tokenization. ex: "352099001761481"
    - wallet_instance_id [string]: unique id refered to the wallet app in the current device. ex: "71583be4777eb89aaf0345eebeb82594f096615ed17862d0"
    """

    def __init__(self, card_id=None, wallet_id=None, wallet_name=None, merchant_id=None, id=None,
                external_id=None, tags=None, status=None, updated=None, created=None, activation_code=None,
                method_code=None, device_type=None, device_name=None, device_serial_number=None, device_os_name=None,
                device_os_version=None, device_imei=None, wallet_instance_id=None):
        Resource.__init__(self, id=id)

        self.card_id = card_id
        self.wallet_id = wallet_id
        self.wallet_name = wallet_name
        self.merchant_id = merchant_id
        self.external_id = external_id
        self.tags = tags
        self.status = status
        self.updated = updated
        self.created = created
        self.activation_code = activation_code
        self.method_code = method_code
        self.device_type = device_type
        self.device_name = device_name
        self.device_serial_number = device_serial_number
        self.device_os_name = device_os_name
        self.device_os_version = device_os_version
        self.device_imei = device_imei
        self.wallet_instance_id = wallet_instance_id


_resource = {"class": IssuingToken, "name": "IssuingToken"}


def get(id, user=None):
    """# Retrieve a specific IssuingToken
    Receive a single IssuingToken object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - IssuingToken object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, status=None, card_ids=None, tags=None, ids=None, user=None, external_ids=None):
    """# Retrieve IssuingTokens
    Receive a generator of IssuingToken objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: current IssuingToken status. ex: "active", "blocked", "canceled", "frozen" or "pending"
    - card_ids [list of strings, default None]: list of card_ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - tags [list of strings, default None]: list of strings for tagging. ex: ["travel", "food"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    - external_ids [list of strings, default None]: external IDs. ex: ["DSHRMC00002626944b0e3b539d4d459281bdba90c2588791", "DSHRMC00002626941c531164a0b14c66ad9602ee716f1e85"]
    ## Return:
    - generator of IssuingToken objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        card_ids=card_ids,
        tags=tags,
        ids=ids,
        user=user,
        external_ids=external_ids,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, card_ids=None, tags=None, ids=None, user=None, external_ids=None):
    """# Retrieve paged IssuingTokens
    Receive a list of IssuingToken objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: current IssuingToken status. ex: "active", "blocked", "canceled", "frozen" or "pending"
    - card_ids [list of strings, default None]: list of card_ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - tags [list of strings, default None]: list of strings for tagging. ex: ["travel", "food"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    - external_ids [list of strings, default None]: external IDs. ex: ["5656565656565656", "4545454545454545"]
    ## Return:
    - list of IssuingToken objects with updated attributes
    - cursor to retrieve the next page of IssuingToken objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        card_ids=card_ids,
        tags=tags,
        ids=ids,
        user=user,
        external_ids=external_ids,
    )


def update(id, status=None, tags=None, user=None):
    """# Update IssuingToken entity
    Update an IssuingToken by passing id.
    ## Parameters (required):
    - id [string]: IssuingToken id. ex: "5656565656565656"
    ## Parameters (optional):
    - status [string, default None]: You may block the IssuingToken by passing "blocked" or activate by passing "active" in the status. ex: "active", "blocked"
    - tags [list of strings, default None]: list of strings for tagging. ex: ["travel", "food"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - target IssuingToken with updated attributes
    """
    payload = {
        "status": status,
        "tags": tags
    }
    return rest.patch_id(resource=_resource, id=id, payload=payload, user=user)    


def cancel(id, user=None):
    """# Cancel an IssuingToken entity
    Cancel an entity previously created in the Stark Infra API by its ID
    ## Parameters (required):
    - id [string]: IssuingToken unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - canceled IssuingToken object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)


def parse(content, signature, user=None):
    """# Create a single verified IssuingToken request from a content string
    Use this method to parse and verify the authenticity of the request received at the informed endpoint.
    Token requests are posted to your registered endpoint whenever IssuingTokens are received.
    If the provided digital signature does not check out with the StarkInfra public key, a stark.exception.InvalidSignatureException will be raised.
    ## Parameters (required):
    - content [string]: response content from request received at user endpoint (not parsed)
    - signature [string]: base-64 digital signature received at response header "Digital-Signature"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - Parsed IssuingToken object
    """
    return parse_and_verify(
        content=content,
        signature=signature,
        user=user,
        resource=_resource,
        key="",
    )


def response_authorization(status, reason=None, activation_methods=None, design_id=None, tags=None):
    """# Helps you respond IssuingToken authorization requests
    When a new tokenization is triggered by your user, a POST request will be made to your registered URL to get your decision to complete the tokenization.
    The POST request must be answered in the following format, within 2 seconds, and with an HTTP status code 200.
    ## Parameters (required):
    - status [string]: sub-issuer response to the authorization. ex: "approved" or "denied"
    ## Parameters (conditionally required):
    - reason [string, default ""]: denial reason. Options: "other", "bruteForce", "subIssuerError", "lostCard", "invalidCard", "invalidHolder", "expiredCard", "canceledCard", "blockedCard", "invalidExpiration", "invalidSecurityCode", "missingTokenAuthorizationUrl", "maxCardTriesExceeded", "maxWalletInstanceTriesExceeded"
    - activation_methods [list of dictionaries, default None]: list of dictionaries with "type":string and "value":string pairs
    - design_id [string, default None]: design unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - tags [list of strings, default None]: tags to filter retrieved object. ex: ["tony", "stark"]
    ## Return:
    - Dumped JSON string that must be returned to us on the IssuingToken request
    """
    params = {"authorization": {
        "status": status,
        "reason": reason or "",
        "activationMethods": activation_methods,
        "designId": design_id,
        "tags": tags,
    }}
    return dumps(api_json(params))


def response_activation(status, reason=None, tags=None):
    """# Helps you respond IssuingToken activation requests
    When a new token activation is triggered by your user, a POST request will be made to your registered URL for you to confirm the activation code you informed to them. You may identify this request through the present activation_code in the payload.
    The POST request must be answered in the following format, within 2 seconds, and with an HTTP status code 200.
    ## Parameters (required):
    - status [string]: sub-issuer response to the activation. ex: "approved" or "denied"
    ## Parameters (optional):
    - reason [string, default ""]: denial reason. Options: "other", "bruteForce", "subIssuerError", "lostCard", "invalidCard", "invalidHolder", "expiredCard", "canceledCard", "blockedCard", "invalidExpiration", "invalidSecurityCode", "missingTokenAuthorizationUrl", "maxCardTriesExceeded", "maxWalletInstanceTriesExceeded"
    - tags [list of strings, default None]: tags to filter retrieved object. ex: ["tony", "stark"]
    ## Return:
    - Dumped JSON string that must be returned to us on the IssuingToken request
    """
    params = {"authorization": {
        "status": status,
        "reason": reason or "",
        "tags": tags,
    }}
    return dumps(api_json(params))



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..utils import rest


class IssuingWithdrawal(Resource):
    """# IssuingWithdrawal object
    The IssuingWithdrawal objects created in your Workspace return cash from your Issuing balance to your
    Banking balance.
    ## Parameters (required):
    - amount [integer]: IssuingWithdrawal value in cents. Minimum = 0 (any value will be accepted). ex: 1234 (= R$ 12.34)
    - external_id [string] IssuingWithdrawal external ID. ex: "12345"
    - description [string]: IssuingWithdrawal description. ex: "sending money back"
    ## Parameters (optional):
    - tags [list of strings, default None]: list of strings for tagging. ex: ["tony", "stark"]
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingWithdrawal is created. ex: "5656565656565656"
    - transaction_id [string]: Stark Bank ledger transaction ids linked to this IssuingWithdrawal
    - issuing_transaction_id [string]: issuing ledger transaction ids linked to this IssuingWithdrawal
    - updated [datetime.datetime]: latest update datetime for the IssuingWithdrawal. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingWithdrawal. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, external_id, description, tags=None, id=None, transaction_id=None,
                 issuing_transaction_id=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.external_id = external_id
        self.description = description
        self.tags = tags
        self.transaction_id = transaction_id
        self.issuing_transaction_id = issuing_transaction_id
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": IssuingWithdrawal, "name": "IssuingWithdrawal"}


def create(withdrawal, user=None):
    """# Create an IssuingWithdrawal
    Send a single IssuingWithdrawal object for creation at the Stark Infra API
    ## Parameters (required):
    - withdrawal [IssuingWithdrawal object]: IssuingWithdrawal object to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingWithdrawal object with updated attributes
    """
    return rest.post_single(resource=_resource, entity=withdrawal, user=user)


def get(id, user=None):
    """# Retrieve a specific IssuingWithdrawal
    Receive a single IssuingWithdrawal object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingWithdrawal object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(external_ids=None, after=None, before=None, limit=None, tags=None, user=None):
    """# Retrieve IssuingWithdrawals
    Receive a generator of IssuingWithdrawal objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - external_ids [list of strings, default None]: external IDs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of IssuingWithdrawal objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        external_ids=external_ids,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        limit=limit,
        user=user,
    )


def page(external_ids=None, after=None, before=None, limit=None, tags=None, cursor=None, user=None):
    """# Retrieve paged IssuingWithdrawals
    Receive a list of IssuingWithdrawal objects previously created in the Stark Infra API and the cursor to the next page.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - external_ids [list of strings, default None]: external IDs. ex: ["5656565656565656", "4545454545454545"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingWithdrawal objects with updated attributes
    - cursor to retrieve the next page of IssuingWithdrawal objects
    """
    return rest.get_page(
        resource=_resource,
        external_ids=external_ids,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        limit=limit,
        cursor=cursor,
        user=user,
    )



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime_or_date


class BrcodePreview(Resource):
    """# BrcodePreview object
    The BrcodePreview object is used to preview information from a BR Code before paying it.
    When you initialize a BrcodePreview, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the created object.
    ## Parameters (required):
    - id [string]: BR Code from a Pix payment. This is also de information directly encoded in a QR Code. ex: "00020126580014br.gov.bcb.pix0136a629532e-7693-4846-852d-1bbff817b5a8520400005303986540510.005802BR5908T'Challa6009Sao Paulo62090505123456304B14A"
    - payer_id [string]: tax id (CPF/CNPJ) of the individual or business requesting the PixKey information. This id is used by the Central Bank to limit request rates. ex: "20.018.183/0001-80"
    ## Parameters (optional):
    - end_to_end_id [string]: central bank's unique transaction ID. ex: "E79457883202101262140HHX553UPqeq"
    ## Attributes (return-only):
    - account_number [string]: Payment receiver account number. ex: "1234567"
    - account_type [string]: Payment receiver account type. ex: "checking"
    - amount [integer]: Value in cents that this payment is expecting to receive. If 0, any value is accepted. ex: 123 (= R$1,23)
    - amount_type [string]: amount type of the Brcode. If the amount type is "custom" the Brcode's amount can be changed by the sender at the moment of payment. Options: "fixed" or "custom"
    - bank_code [string]: Payment receiver bank code. ex: "20018183"
    - branch_code [string]: Payment receiver branch code. ex: "0001"
    - cash_amount [integer]: Amount to be withdrawn from the cashier in cents. ex: 1000 (= R$ 10.00)
    - cashier_bank_code [string]: Cashier's bank code. ex: "20018183"
    - cashier_type [string]: Cashier's type. Options: "merchant", "participant" and "other"
    - discount_amount [integer]: Discount value calculated over nominal_amount. ex: 3000
    - fine_amount [integer]: Fine value calculated over nominal_amount. ex: 20000
    - interest_amount [integer]: Interest value calculated over nominal_amount. ex: 10000
    - key_id [string]: Receiver's PixKey id. ex: "+5511989898989"
    - name [string]: Payment receiver name. ex: "Tony Stark"
    - nominal_amount [integer]: Brcode emission amount, without fines, fees and discounts. ex: 1234 (= R$ 12.34)
    - reconciliation_id [string]: Reconciliation ID linked to this payment. If the brcode is dynamic, the reconciliation_id will have from 26 to 35 alphanumeric characters, ex: "cd65c78aeb6543eaaa0170f68bd741ee". If the brcode is static, the reconciliation_id will have up to 25 alphanumeric characters "ah27s53agj6493hjds6836v49"
    - reduction_amount [integer]: Reduction value to discount from nominal_amount. ex: 1000
    - scheduled [datetime.datetime]: date of payment execution. ex: datetime(2020, 3, 10)
    - status [string]: Payment status. ex: "active", "paid", "canceled" or "unknown"
    - tax_id [string]: Payment receiver tax ID. ex: "012.345.678-90"
    """

    def __init__(self, id, payer_id, account_number=None, account_type=None, amount=None, amount_type=None, bank_code=None,
                 branch_code=None, cash_amount=None, cashier_bank_code=None, cashier_type=None, discount_amount=None,
                 fine_amount=None, interest_amount=None, key_id=None, name=None, nominal_amount=None, end_to_end_id=None,
                 reconciliation_id=None, reduction_amount=None, scheduled=None, status=None, tax_id=None):
        Resource.__init__(self, id=id)
        
        self.payer_id = payer_id
        self.end_to_end_id = end_to_end_id
        self.account_number = account_number
        self.account_type = account_type
        self.amount = amount
        self.amount_type = amount_type
        self.bank_code = bank_code
        self.branch_code = branch_code
        self.cash_amount = cash_amount
        self.cashier_bank_code = cashier_bank_code
        self.cashier_type = cashier_type
        self.discount_amount = discount_amount
        self.fine_amount = fine_amount
        self.interest_amount = interest_amount
        self.key_id = key_id
        self.name = name
        self.nominal_amount = nominal_amount
        self.reconciliation_id = reconciliation_id
        self.reduction_amount = reduction_amount
        self.scheduled = check_datetime_or_date(scheduled)
        self.status = status
        self.tax_id = tax_id


_resource = {"class": BrcodePreview, "name": "BrcodePreview"}


def create(previews, user=None):
    """# Retrieve BrcodePreviews
    Process BR Codes before paying them.
    ## Parameters (required):
    - previews [list of BrcodePreview objects]: List of BrcodePreview objects to preview. ex: [starkinfra.BrcodePreview("00020126580014br.gov.bcb.pix0136a629532e-7693-4846-852d-1bbff817b5a8520400005303986540510.005802BR5908T'Challa6009Sao Paulo62090505123456304B14A")]
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of BrcodePreview objects with updated attributes
    """
    return rest.post_multi(
        resource=_resource,
        entities=previews,
        user=user,
    )



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date, check_datetime
from ..utils import rest


class IssuingEmbossingRequest(Resource):
    """# IssuingEmbossingRequest object
    The IssuingEmbossingRequest object displays the information of embossing requests in your Workspace.
    ## Parameters (required):
    - card_id [string]: id of the IssuingCard to be embossed. ex "5656565656565656"
    - kit_id [string]: card embossing kit id. ex "5656565656565656"
    - display_name_1 [string]: card displayed name. ex: "ANTHONY STARK"
    - shipping_city [string]: shipping city. ex: "NEW YORK"
    - shipping_country_code [string]: shipping country code. ex: "US"
    - shipping_district [string]: shipping district. ex: "NY"
    - shipping_state_code [string]: shipping state code. ex: "NY"
    - shipping_street_line_1 [string]: shipping main address. ex: "AVENUE OF THE AMERICAS"
    - shipping_street_line_2 [string]: shipping address complement. ex: "Apt. 6"
    - shipping_service [string]: shipping service. ex: "loggi"
    - shipping_tracking_number [string]: shipping tracking number. ex: "5656565656565656"
    - shipping_zip_code [string]: shipping zip code. ex: "12345-678"
    ## Parameters (optional):
    - embosser_id [string]: id of the card embosser. ex: "5656565656565656"
    - display_name_2 [string]: card displayed name. ex: "IT Services"
    - display_name_3 [string]: card displayed name. ex: "StarkBank S.A."
    - shipping_phone [string]: shipping phone. ex: "+5511999999999"
    - tags [list of strings, default None]: list of strings for tagging. ex: ["card", "corporate"]
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingEmbossingRequest is created. ex: "5656565656565656"
    - fee [integer]: fee charged when IssuingEmbossingRequest is created. ex: 1000
    - status [string]: status of the IssuingEmbossingRequest. ex: "created", "processing", "success", "failed"
    - updated [datetime.datetime]: latest update datetime for the IssuingEmbossingRequest. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingEmbossingRequest. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, card_id, kit_id, display_name_1, shipping_city,
                 shipping_country_code, shipping_district, shipping_state_code, shipping_street_line_1, 
                 shipping_street_line_2, shipping_service, shipping_tracking_number, shipping_zip_code, 
                 embosser_id=None, display_name_2=None, display_name_3=None, shipping_phone=None, 
                 tags=None, id=None, fee=None, status=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.card_id = card_id
        self.kit_id = kit_id
        self.display_name_1 = display_name_1
        self.shipping_city = shipping_city
        self.shipping_country_code = shipping_country_code
        self.shipping_district = shipping_district
        self.shipping_state_code = shipping_state_code
        self.shipping_street_line_1 = shipping_street_line_1
        self.shipping_street_line_2 = shipping_street_line_2
        self.shipping_service = shipping_service
        self.shipping_tracking_number = shipping_tracking_number
        self.shipping_zip_code = shipping_zip_code
        self.embosser_id = embosser_id
        self.display_name_2 = display_name_2
        self.display_name_3 = display_name_3
        self.shipping_phone = shipping_phone
        self.tags = tags
        self.fee = fee
        self.status = status
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": IssuingEmbossingRequest, "name": "IssuingEmbossingRequest"}


def create(requests, user=None):
    """# Create IssuingEmbossingRequests
    Send a list of IssuingEmbossingRequest objects for creation at the Stark Infra API
    ## Parameters (required):
    - requests [list of IssuingEmbossingRequest objects]: list of IssuingEmbossingRequest objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingEmbossingRequest objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=requests, user=user)


def query(limit=None, after=None, before=None, status=None, card_ids=None, ids=None, tags=None, user=None):
    """# Retrieve IssuingEmbossingRequests
    Receive a generator of IssuingEmbossingRequest objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "processing", "success", "failed"]
    - card_ids [list of string, default None]: list of card_ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of IssuingEmbossingRequest objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        card_ids=card_ids,
        ids=ids,
        tags=tags,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, card_ids=None, ids=None, tags=None, user=None):
    """# Retrieve paged IssuingEmbossingRequests
    Receive a list of up to 100 IssuingEmbossingRequest objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "processing", "success", "failed"]
    - card_ids [list of string, default None]: list of card_ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingEmbossingRequest objects with updated attributes
    - cursor to retrieve the next page of IssuingEmbossingRequest objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        card_ids=card_ids,
        ids=ids,
        tags=tags,
        user=user,
    )


def get(id, user=None):
    """# Retrieve a specific IssuingEmbossingRequest
    Receive a single IssuingEmbossingRequest object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingEmbossingRequest object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)



from starkcore.utils.subresource import SubResource


class Certificate(SubResource):
    """# pixdomain.Certificate object
    The Certificate object displays the certificate information from a specific domain.
    ## Attributes (return-only):
    - content [string]: certificate of the Pix participant in PEM format.
    """

    def __init__(self, content=None):
        self.content = content


_resource = {"class": Certificate, "name": "Certificate"}



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date, check_datetime
from ..utils import rest
from ..issuingrule import parse_rules


class IssuingCard(Resource):
    """# IssuingCard object
    The IssuingCard object displays the information of the cards created in your Workspace.
    Sensitive information will only be returned when the "expand" parameter is used, to avoid security concerns.
    ## Parameters (required):
    - holder_name [string]: card holder name. ex: "Tony Stark"
    - holder_tax_id [string]: card holder tax ID. ex: "012.345.678-90"
    - holder_external_id [string]: card holder unique id, generated by the user to avoid duplicated holders. ex: "my-entity/123"
    ## Parameters (optional):
    - display_name [string, default None]: card displayed name. ex: "ANTHONY STARK"
    - rules [list of IssuingRule, default []]: [EXPANDABLE] list of card spending rules.
    - product_id [string, default None]: card product ID to which the card is bound. ex: "53810200"
    - tags [list of strings, default []]: list of strings for tagging. ex: ["travel", "food"]
    - street_line_1 [string, default sub-issuer street line 1]: card holder main address. ex: "Av. Paulista, 200"
    - street_line_2 [string, default sub-issuer street line 2]: card holder address complement. ex: "Apto. 123"
    - district [string, default sub-issuer district]: card holder address district / neighbourhood. ex: "Bela Vista"
    - city [string, default sub-issuer city]: card holder address city. ex: "Rio de Janeiro"
    - state_code [string, default sub-issuer state code]: card holder address state. ex: "GO"
    - zip_code [string, default sub-issuer zip code]: card holder address zip code. ex: "01311-200"
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingCard is created. ex: "5656565656565656"
    - holder_id [string]: card holder unique id. ex: "5656565656565656"
    - type [string]: card type. ex: "virtual"
    - status [string]: current IssuingCard status. ex: "active", "blocked", "canceled", "expired"
    - number [string]: [EXPANDABLE] masked card number. Expand to unmask the value. ex: "123"
    - security_code [string]: [EXPANDABLE] masked card verification value (cvv). Expand to unmask the value. ex: "123"
    - expiration [datetime.datetime]: [EXPANDABLE] masked card expiration datetime. Expand to unmask the value. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the IssuingCard. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingCard. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, holder_name, holder_tax_id, holder_external_id, display_name=None, rules=None, product_id=None,
                 tags=None, street_line_1=None, street_line_2=None, district=None, city=None, state_code=None,
                 zip_code=None, id=None, holder_id=None, type=None, status=None, number=None, security_code=None,
                 expiration=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.holder_name = holder_name
        self.holder_tax_id = holder_tax_id
        self.holder_external_id = holder_external_id
        self.display_name = display_name
        self.rules = parse_rules(rules)
        self.product_id = product_id
        self.tags = tags
        self.street_line_1 = street_line_1
        self.street_line_2 = street_line_2
        self.district = district
        self.city = city
        self.state_code = state_code
        self.zip_code = zip_code
        self.holder_id = holder_id
        self.type = type
        self.status = status
        self.number = number
        self.security_code = security_code
        self.expiration = check_datetime(expiration)
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": IssuingCard, "name": "IssuingCard"}


def create(cards, expand=None, user=None):
    """# Create IssuingCards
    Send a list of IssuingCard objects for creation at the Stark Infra API
    ## Parameters (required):
    - cards [list of IssuingCard objects]: list of IssuingCard objects to be created in the API
    ## Parameters (optional):
    - expand [list of strings, default None]: fields to expand information. ex: ["rules", "security_code", "number", "expiration"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingCard objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=cards, expand=expand, user=user)


def query(limit=None, ids=None, after=None, before=None, status=None, types=None, holder_ids=None, tags=None,
          expand=None, user=None):
    """# Retrieve IssuingCards
    Receive a generator of IssuingCard objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["active", "blocked", "canceled", "expired"]
    - types [list of strings, default None]: card type. ex: ["virtual"]
    - holder_ids [list of strings]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - expand [list of strings, default None]: fields to expand information. ex: ["rules", "security_code", "number", "expiration"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of IssuingCard objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        ids=ids,
        after=check_date(after),
        before=check_date(before),
        status=status,
        types=types,
        holder_ids=holder_ids,
        tags=tags,
        expand=expand,
        user=user,
    )


def page(cursor=None, limit=None, ids=None, after=None, before=None, status=None, types=None, holder_ids=None,
         tags=None, expand=None, user=None):
    """# Retrieve paged IssuingCards
    Receive a list of IssuingCard objects previously created in the Stark Infra API and the cursor to the next page.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["active", "blocked", "canceled", "expired"]
    - types [list of strings, default None]: card type. ex: ["virtual"]
    - holder_ids [list of strings]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - expand [list of strings, default None]: fields to expand information. ex: ["rules", "security_code", "number", "expiration"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingCard objects with updated attributes
    - cursor to retrieve the next page of IssuingCard objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        ids=ids,
        after=check_date(after),
        before=check_date(before),
        status=status,
        types=types,
        holder_ids=holder_ids,
        tags=tags,
        expand=expand,
        user=user,
    )


def get(id, expand=None, user=None):
    """# Retrieve a specific IssuingCard
    Receive a single IssuingCard object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - expand [list of strings, default None]: fields to expand information. ex: ["rules", "security_code", "number", "expiration"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingCards object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, expand=expand, user=user)


def update(id, status=None, pin=None, display_name=None, rules=None, tags=None, user=None):
    """# Update IssuingCard entity
    Update an IssuingCard by passing id.
    ## Parameters (required):
    - id [string]: IssuingCard id. ex: '5656565656565656'
    ## Parameters (optional):
    - status [string, default None]: You may block the IssuingCard by passing 'blocked' or activate by passing 'active' in the status
    - pin [string, default None]: You may unlock your physical card by passing its PIN. This is also the PIN you use to authorize a purhcase.
    - display_name [string, default None]: card displayed name. ex: "ANTHONY EDWARD"
    - rules [list of dictionaries, default None]: list of dictionaries with "amount": int, "currencyCode": string, "id": string, "interval": string, "name": string pairs.
    - tags [list of strings, default None]: list of strings for tagging
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - target IssuingCard with updated attributes
    """
    payload = {
        "status": status,
        "display_name": display_name,
        "pin": pin,
        "rules": rules,
        "tags": tags,
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def cancel(id, user=None):
    """# Cancel an IssuingCard entity
    Cancel an IssuingCard entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: IssuingCard unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - canceled IssuingCard object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from starkcore.utils.subresource import SubResource
from starkinfra.utils import rest


class MerchantCountry(SubResource):
    """# MerchantCountry object
    MerchantCountry's codes are used to define countries filters in IssuingRules.
    ## Parameters (required):
    - code [string]: country's code. ex: "BRA"
    ## Attributes (return-only):
    - name [string]: country's name. ex: "Brazil"
    - number [string]: country's number. ex: "076"
    - short_code [string]: country's short code. ex: "BR"
    """

    def __init__(self, code, name=None, number=None, short_code=None):
        self.code = code
        self.name = name
        self.number = number
        self.short_code = short_code


_resource = {"class": MerchantCountry, "name": "MerchantCountry"}


def query(search=None, user=None):
    """# Retrieve MerchantCountries
    Receive a generator of MerchantCountry objects previously created in the Stark Infra API
    ## Parameters (optional):
    - search [string, default None]: keyword to search for code, name, number or short_code
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of MerchantCountry objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        search=search,
        user=user,
    )



from json import dumps
from ..utils import rest
from ..utils import parse
from starkcore.utils.api import api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class DynamicBrcode(Resource):
    """# DynamicBrcode object
    BR codes store information represented by Pix QR Codes, which are used to
    send or receive Pix transactions in a convenient way.
    DynamicBrcodes represent charges with information that can change at any time,
    since all data needed for the payment is requested dynamically to an URL stored
    in the BR Code. Stark Infra will receive the GET request and forward it to your
    registered endpoint with a GET request containing the UUID of the BR code for
    identification.
    When you initialize a DynamicBrcode, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the created object.
    ## Parameters (required):
    - name [string]: receiver's name. ex: "Tony Stark"
    - city [string]: receiver's city name. ex: "Rio de Janeiro"
    - external_id [string]: string that must be unique among all your DynamicBrcodes. Duplicated external ids will cause failures. ex: "my-internal-id-123456"
    ## Parameters (optional):
    - type [string, default "instant"]: type of the DynamicBrcode. Options: "instant", "due"
    - tags [list of strings, default []]: list of strings for tagging. ex: ["travel", "food"]
    ## Attributes (return-only):
    - id [string]: id returned on creation, this is the BR code. ex: "00020126360014br.gov.bcb.pix0114+552840092118152040000530398654040.095802BR5915Jamie Lannister6009Sao Paulo620705038566304FC6C"
    - uuid [string]: unique uuid returned when the DynamicBrcode is created. ex: "4e2eab725ddd495f9c98ffd97440702d"
    - url [string]: url link to the BR code image. ex: "https://brcode-h.development.starkinfra.com/dynamic-qrcode/901e71f2447c43c886f58366a5432c4b.png"
    - updated [datetime.datetime]: latest update datetime for the DynamicBrcode. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the DynamicBrcode. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, name, city, external_id, id=None, type=None, tags=None, uuid=None, url=None, 
                    updated=None, created=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.city = city
        self.external_id = external_id
        self.type = type
        self.tags = tags
        self.uuid = uuid
        self.url = url
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": DynamicBrcode, "name": "DynamicBrcode"}


def create(brcodes, user=None):
    """# Create DynamicBrcodes
    Send a list of DynamicBrcode objects for creation at the Stark Infra API
    ## Parameters (required):
    - brcodes [list of DynamicBrcode objects]: list of DynamicBrcode objects to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of DynamicBrcode objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=brcodes, user=user)


def get(uuid, user=None):
    """# Retrieve a specific DynamicBrcode
    Receive a single DynamicBrcode object previously created in the Stark Infra API by its uuid
    ## Parameters (required):
    - uuid [string]: object's unique uuid. ex: "901e71f2447c43c886f58366a5432c4b"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - DynamicBrcode object with updated attributes
    """
    return rest.get_id(resource=_resource, id=uuid, user=user)


def query(limit=None, after=None, before=None, external_id=None, uuids=None, tags=None, user=None):
    """# Retrieve DynamicBrcodes
    Receive a generator of DynamicBrcode objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - external_ids [list of strings, default None]: list of external_ids to filter retrieved objects. ex: ["my_external_id1", "my_external_id2"]
    - uuids [list of strings, default None]: list of uuids to filter retrieved objects. ex: ["901e71f2447c43c886f58366a5432c4b", "4e2eab725ddd495f9c98ffd97440702d"]
    - tags [list of strings, default None]: list of tags to filter retrieved objects. ex: ["travel", "food"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of DynamicBrcode objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        external_id=external_id,
        uuids=uuids,
        tags=tags,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, external_id=None, uuids=None, tags=None, user=None):
    """# Retrieve DynamicBrcodes
    Receive a list of DynamicBrcode objects previously created in the Stark Infra API and the cursor to the next page.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - external_ids [list of strings, default None]: list of external_ids to filter retrieved objects. ex: ["my_external_id1", "my_external_id2"]
    - uuids [list of strings, default None]: list of uuids to filter retrieved objects. ex: ["901e71f2447c43c886f58366a5432c4b", "4e2eab725ddd495f9c98ffd97440702d"]
    - tags [list of strings, default None]: list of tags to filter retrieved objects. ex: ["travel", "food"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of DynamicBrcode objects with updated attributes
    - cursor to retrieve the next page of DynamicBrcode objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        external_id=external_id,
        uuids=uuids,
        tags=tags,
        user=user,
    )


def response_due(version, created, due, key_id, status, reconciliation_id, nominal_amount, sender_name, sender_tax_id,
                 receiver_name, receiver_tax_id, receiver_street_line, receiver_city, receiver_state_code, receiver_zip_code,
                 expiration=None, fine=None, interest=None, discounts=None, description=None):
    """# Helps you respond to a due DynamicBrcode Read
    When a Due DynamicBrcode is read by your user, a GET request containing the Brcode's 
    UUID will be made to your registered URL to retrieve additional information needed 
    to complete the transaction.
    The GET request must be answered in the following format, within 5 seconds, and with
    an HTTP status code 200.
    ## Parameters (required):
    - version [integer]: integer that represents how many times the BR code was updated.
    - created [datetime.datetime or string]: creation datetime in ISO format of the DynamicBrcode. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - due [datetime.datetime or string]: requested payment due datetime in ISO format. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - key_id [string]: receiver's PixKey id. Can be a tax_id (CPF/CNPJ), a phone number, an email or an alphanumeric sequence (EVP). ex: "+5511989898989"
    - status [string]: BR code status. Options: "created", "overdue", "paid", "canceled" or "expired"
    - reconciliation_id [string]: id to be used for conciliation of the resulting Pix transaction. This id must have from to 26 to 35 alphanumeric characters ex: "cd65c78aeb6543eaaa0170f68bd741ee"
    - nominal_amount [integer]: positive integer that represents the amount in cents of the resulting Pix transaction. ex: 1234 (= R$ 12.34)
    - sender_name [string]: sender's full name. ex: "Anthony Edward Stark"
    - sender_tax_id [string]: sender's CPF (11 digits formatted or unformatted) or CNPJ (14 digits formatted or unformatted). ex: "01.001.001/0001-01"
    - receiver_name [string]: receiver's full name. ex: "Jamie Lannister"
    - receiver_tax_id [string]: receiver's CPF (11 digits formatted or unformatted) or CNPJ (14 digits formatted or unformatted). ex: "012.345.678-90"
    - receiver_street_line [string]: receiver's main address. ex: "Av. Paulista, 200"
    - receiver_city [string]: receiver's address city name. ex: "Sao Paulo"
    - receiver_state_code [string]: receiver's address state code. ex: "SP"
    - receiver_zip_code [string]: receiver's address zip code. ex: "01234-567"
    ## Parameters (optional):
    - expiration [datime.timedelta or integer, default 86400 (1 day)]: time in seconds counted from the creation datetime until the DynamicBrcode expires. After expiration, the BR code cannot be paid anymore.
    - fine [float, default 2.0]: Percentage charged if the sender pays after the due datetime.
    - interest [float, default 1.0]: Interest percentage charged if the sender pays after the due datetime.
    - discounts [list of dictionaries, default None]: list of dictionaries with "percentage":float and "due":date.datetime or string pairs.
    - description [string, default None]: additional information to be shown to the sender at the moment of payment.
    ## Return:
    - Dumped JSON string that must be returned to us
    """
    params = {
        "version": version,
        "created": created,
        "due": due,
        "keyId": key_id,
        "status": status,
        "reconciliationId": reconciliation_id,
        "nominalAmount": nominal_amount,
        "senderName": sender_name,
        "senderTaxId": sender_tax_id,
        "receiverName": receiver_name,
        "receiverTaxId": receiver_tax_id,
        "receiverStreetLine": receiver_street_line,
        "receiverCity": receiver_city,
        "receiverStateCode": receiver_state_code,
        "receiverZipCode": receiver_zip_code,
        "expiration": expiration,
        "fine": fine,
        "interest": interest,
        "discounts": discounts,
        "description": description,
    }
    return dumps(api_json(params))


def response_instant(version, created, key_id, status, reconciliation_id, amount, expiration=None, sender_name=None, sender_tax_id=None,
                    description=None, amount_type=None, cash_amount=None, cashier_type=None, cashier_bank_code=None):
    """# Helps you respond to an instant DynamicBrcode Read
    When an instant DynamicBrcode is read by your user, a GET request containing the BR code's UUID will be made
    to your registered URL to retrieve additional information needed to complete the transaction.
    The GET request must be answered in the following format within 5 seconds and with an HTTP status code 200.
    ## Parameters (required):
    - version [integer]: integer that represents how many times the BR code was updated.
    - created [datetime.datetime or string]: creation datetime of the DynamicBrcode. ex: "2022-05-17"
    - key_id [string]: receiver's PixKey id. Can be a tax_id (CPF/CNPJ), a phone number, an email or an alphanumeric sequence (EVP). ex: "+5511989898989"
    - status [string]: BR code status. Options: "created", "overdue", "paid", "canceled" or "expired"
    - reconciliation_id [string]: id to be used for conciliation of the resulting Pix transaction. ex: "cd65c78aeb6543eaaa0170f68bd741ee"
    - amount [integer]: positive integer that represents the amount in cents of the resulting Pix transaction. ex: 1234 (= R$ 12.34)
    ## Parameters (conditionally-required):
    - cashier_type [string, default None]: cashier's type. Required if the cashAmount is different from 0. Options: "merchant", "participant" and "other"
    - cashier_bank_code [string, default None]: cashier's bank code. Required if the cashAmount is different from 0. ex: "20018183"
    ## Parameters (optional):
    - cash_amount [integer, default 0]: amount to be withdrawn from the cashier in cents. ex: 1000 (= R$ 10.00)
    - expiration [datetime.timedelta or integer, default 86400 (1 day)]: time in seconds counted from the creation datetime until the DynamicBrcode expires. After expiration, the BR code cannot be paid anymore. Default value: 86400 (1 day)
    - sender_name [string, default None]: sender's full name. ex: "Anthony Edward Stark"
    - sender_tax_id [string, default None]: sender's CPF (11 digits formatted or unformatted) or CNPJ (14 digits formatted or unformatted). ex: "01.001.001/0001-01"
    - amount_type [string, default "fixed"]: amount type of the Brcode. If the amount type is "custom" the Brcode's amount can be changed by the sender at the moment of payment. Options: "fixed"or "custom"
    - description [string, default None]: additional information to be shown to the sender at the moment of payment.
    ## Return:
    - Dumped JSON string that must be returned to us
    """
    params = {
        "version": version,
        "created": created,
        "keyId": key_id,
        "status": status,
        "reconciliationId": reconciliation_id,
        "amount": amount,
        "cashierType": cashier_type,
        "cashierBankCode": cashier_bank_code,
        "cashAmount": cash_amount,
        "expiration": expiration,
        "senderName": sender_name,
        "senderTaxId": sender_tax_id,
        "amountType": amount_type,
        "description": description,
    }
    return dumps(api_json(params))


def verify(uuid, signature, user=None):
    """# Verify a DynamicBrcode Read 
    When a DynamicBrcode is read by your user, a GET request will be made to your registered URL to
    retrieve additional information needed to complete the transaction.
    Use this method to verify the authenticity of a GET request received at your registered endpoint.
    If the provided digital signature does not check out with the StarkInfra public key,
    a stark.exception.InvalidSignatureException will be raised.
    ## Parameters (required):
    - uuid [string]: unique uuid returned when a DynamicBrcode is created. ex: "4e2eab725ddd495f9c98ffd97440702d"
    - signature [string]: base-64 digital signature received at response header "Digital-Signature"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - verified Brcode's uuid.
    """
    return parse.verify(
        content=uuid,
        signature=signature,
        user=user,
    )



from starkcore.utils.subresource import SubResource
from starkinfra.utils import rest


class IssuingTokenRequest(SubResource):
    """# IssuingTokenRequest object
    The IssuingTokenRequest object displays the necessary information to proceed with the card tokenization.
    ## Parameters (required):
    - card_id [string]: card id to be tokenized. ex: "5734340247945216"
    - wallet_id [string]: desired wallet to be integrated. ex: "google"
    - method_code [string]: method code. ex: "app" or "manual"
    ## Attributes (return-only):
    - content [string]: token request content. ex: "eyJwdWJsaWNLZXlGaW5nZXJwcmludCI6ICJlNTNiZThjZTRhYWQxNWU2OWNmMjExOTA5Mjk4YzJkOTE0O..."
    - signature [string]: token request signature. ex: "eyJwdWJsaWNLZXlGaW5nZXJwcmludCI6ICJlNTNiZThjZTRhYWQxNWU2OWNmMjExOTA5Mjk4YzJkOTE0O..."
    - metadata [dictionary]: dictionary object used to store additional information about the IssuingTokenRequest object.
    """

    def __init__(self, card_id, wallet_id, method_code, content=None, signature=None, metadata=None):
        self.card_id = card_id
        self.wallet_id = wallet_id
        self.method_code = method_code
        self.content = content
        self.signature = signature
        self.metadata = metadata


_resource = {"class": IssuingTokenRequest, "name": "IssuingTokenRequest"}


def create(request, user=None):
    """# Create an IssuingTokenRequest object
    Send an IssuingTokenRequest object to Stark Infra API to create the payload to proceed with the card tokenization
    ## Parameters (required):
    - request [IssuingTokenRequest object]: IssuingTokenRequest object to the API to generate the payload
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - IssuingTokenRequest object with updated attributes
    """
    return rest.post_single(resource=_resource, entity=request, user=user)



from ..utils import rest
from .__certificate import _resource as _certificate_resource
from starkcore.utils.api import from_api_json
from starkcore.utils.subresource import SubResource


class PixDomain(SubResource):
    """# PixDomain object
    The PixDomain object displays the domain name and the QR Code domain certificate of Pix participants.
    All certificates must be registered with the Central Bank.
    ## Attributes (return-only):
    - certificates [list of pixdomain.Certificate]: certificate information of the Pix participant.
    - name [string]: current active domain (URL) of the Pix participant.
    """

    def __init__(self, certificates=None, name=None):
        self.certificates = _parse_certificates(certificates)
        self.name = name


def _parse_certificates(certificates):
    return [from_api_json(_certificate_resource, certificate) for certificate in certificates]


_resource = {"class": PixDomain, "name": "PixDomain"}


def query(user=None):
    """# Retrieve PixDomains
    Receive a generator of PixDomain objects.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of PixDomain objects with updated attributes
    """
    return rest.get_stream(resource=_resource, user=user)



from starkcore.utils.subresource import SubResource


class Discount(SubResource):
    """# creditnote.invoice.Discount object
    Invoice discount information.
    ## Parameters (required):
    - percentage [float]: percentage of discount applied until specified due date
    - due [datetime.datetime or string]: due datetime for the discount
    """

    def __init__(self, percentage, due):
        self.percentage = percentage
        self.due = due


resource = {"class": Discount, "name": "Discount"}



from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkinfra.cardmethod.__cardmethod import _resource as _method_resource, CardMethod
from starkinfra.merchantcountry.__merchantcountry import _resource as _country_resource, MerchantCountry
from starkinfra.merchantcategory.__merchantcategory import _resource as _category_resource, MerchantCategory


class IssuingRule(Resource):
    """# IssuingRule object
    The IssuingRule object displays the spending rules of IssuingCards and IssuingHolders created in your Workspace.
    ## Parameters (required):
    - name [string]: rule name. ex: "Travel" or "Food"
    - amount [integer]: maximum amount that can be spent in the informed interval. ex: 200000 (= R$ 2000.00)
    ## Parameters (optional):
    - interval [string, default "lifetime"]: interval after which the rule amount counter will be reset to 0. ex: "instant", "day", "week", "month", "year" or "lifetime"
    - currency_code [string, default "BRL"]: code of the currency that the rule amount refers to. ex: "BRL" or "USD"
    - categories [list of MerchantCategories, default []]: merchant categories accepted by the rule. ex: [MerchantCategory(code="fastFoodRestaurants")]
    - countries [list of MerchantCountries, default []]: countries accepted by the rule. ex: [MerchantCountry(code="BRA")]
    - methods [list of CardMethods, default []]: card purchase methods accepted by the rule. ex: [CardMethod(code="magstripe")]
    ## Attributes (expanded return-only):
    - id [string]: unique id returned when an IssuingRule is created, used to update a specific IssuingRule. ex: "5656565656565656"
    - counter_amount [integer]: current rule spent amount. ex: 1000
    - currency_symbol [string]: currency symbol. ex: "R$"
    - currency_name [string]: currency name. ex: "Brazilian Real"
    """

    def __init__(self, name, amount, id=None, interval=None, currency_code=None, categories=None, countries=None,
                 methods=None, counter_amount=None, currency_symbol=None, currency_name=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.amount = amount
        self.interval = interval
        self.currency_code = currency_code
        self.categories = _parse_categories(categories)
        self.countries = _parse_countries(countries)
        self.methods = _parse_methods(methods)
        self.counter_amount = counter_amount
        self.currency_symbol = currency_symbol
        self.currency_name = currency_name


_resource = {"class": IssuingRule, "name": "IssuingRule"}


def _parse_categories(categories):
    if not categories:
        return []
    parsed_categories = []
    for category in categories:
        if isinstance(category, MerchantCategory):
            parsed_categories.append(category)
            continue
        parsed_categories.append(from_api_json(_category_resource, category))
    return parsed_categories


def _parse_countries(countries):
    if not countries:
        return []
    parsed_countries = []
    for country in countries:
        if isinstance(country, MerchantCountry):
            parsed_countries.append(country)
            continue
        parsed_countries.append(from_api_json(_country_resource, country))
    return parsed_countries


def _parse_methods(methods):
    if not methods:
        return []
    parsed_methods = []
    for method in methods:
        if isinstance(method, CardMethod):
            parsed_methods.append(method)
            continue
        parsed_methods.append(from_api_json(_method_resource, method))
    return parsed_methods


def parse_rules(rules):
    parsed_rules = []
    if rules is None:
        return rules
    for rule in rules:
        if isinstance(rule, IssuingRule):
            parsed_rules.append(rule)
            continue
        parsed_rules.append(from_api_json(_resource, rule))
    return parsed_rules

